import os
import unittest

from pychemstation.control import HPLCController
from pychemstation.utils.sequence_types import *
from tests.constants import *


class TestSequence(unittest.TestCase):
    def setUp(self):
        path_constants = room(254)
        for path in path_constants:
            if not os.path.exists(path):
                self.fail(
                    f"{path} does not exist on your system. If you would like to run tests, please change this path.")

        self.hplc_controller = HPLCController(comm_dir=path_constants[0],
                                              method_dir=path_constants[1],
                                              data_dir=path_constants[2],
                                              sequence_dir=path_constants[3])

    def test_switch(self):
        try:
            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
        except Exception as e:
            self.fail(f"Should have not expected: {e}")

    def test_edit_row(self):
        self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
        try:
            self.hplc_controller.edit_sequence_row(SequenceEntry(
                vial_location=TenColumn.TEN,
                method=DEFAULT_METHOD,
                num_inj=3,
                inj_vol=4,
                sample_name="Blank",
                sample_type=SampleType.BLANK,
                inj_source=InjectionSource.HIP_ALS
            ), 1)
        except Exception:
            self.fail("Should have not failed")

    def test_edit_entire_table(self):
        self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
        try:
            seq_table = SequenceTable(
                name=DEFAULT_SEQUENCE,
                rows=[
                    SequenceEntry(
                        vial_location=TenColumn.ONE,
                        method=DEFAULT_METHOD,
                        num_inj=3,
                        inj_vol=4,
                        sample_name="Sampel1",
                        sample_type=SampleType.SAMPLE,
                        inj_source=InjectionSource.HIP_ALS
                    ),
                    SequenceEntry(
                        vial_location=TenColumn.TWO,
                        method=DEFAULT_METHOD,
                        num_inj=3,
                        inj_vol=4,
                        sample_name="Sampel2",
                        sample_type=SampleType.SAMPLE,
                        inj_source=InjectionSource.HIP_ALS
                    )
                ]
            )
            self.hplc_controller.edit_sequence(seq_table)
        except Exception:
            self.fail("Should have not occured")

    def test_load(self):
        try:
            seq = self.hplc_controller.load_sequence()
            self.assertTrue(len(seq.rows) > 0)
        except Exception as e:
            self.fail(f"Should have not expected: {e}")



from dataclasses import dataclass
from enum import Enum
from typing import Union, Any, Optional

from .table_types import RegisterFlag
from ..generated import Signal, SolventElement


class PType(Enum):
    STR = "str"
    NUM = "num"


@dataclass
class Param:
    ptype: PType
    val: Union[float, int, str, Any]
    chemstation_key: Union[RegisterFlag, list[RegisterFlag]]


@dataclass
class HPLCMethodParams:
    organic_modifier: int
    flow: float
    maximum_run_time: int


@dataclass
class TimeTableEntry:
    start_time: float
    organic_modifer: float
    flow: Optional[float] = None


@dataclass
class MethodTimetable:
    first_row: HPLCMethodParams
    subsequent_rows: list[TimeTableEntry]
    dad_wavelengthes: Optional[list[Signal]] = None
    organic_modifier: Optional[SolventElement] = None
    modifier_a: Optional[SolventElement] = None



from .pump_method import (
    Channel1ExtendedSolventType,
    Channel2ExtendedSolventType,
    CompressA,
    CompressB,
    Definition,
    PostTime,
    PumpMethod,
    SolventComposition,
    SolventDescription,
    SolventElement,
    StopTime,
    StrokeA,
    Timetable,
    TimetableEntry,
)

from .dad_method import (
    AnalogOutput1,
    AnalogOutput2,
    DadMethod,
    PostTime,
    PrepareAutomation,
    PrepareMode,
    Signal,
    Signals,
    SpectraAcquisition,
    StopTime,
)

__all__ = [
    "Channel1ExtendedSolventType",
    "Channel2ExtendedSolventType",
    "CompressA",
    "CompressB",
    "Definition",
    "PostTime",
    "PumpMethod",
    "SolventComposition",
    "SolventDescription",
    "SolventElement",
    "StopTime",
    "StrokeA",
    "Timetable",
    "TimetableEntry",
    "AnalogOutput1",
    "AnalogOutput2",
    "DadMethod",
    "PostTime",
    "PrepareAutomation",
    "PrepareMode",
    "Signal",
    "Signals",
    "SpectraAcquisition",
    "StopTime",
]



import os

import unittest

from pychemstation.control import HPLCController
from tests.constants import *

run_too = False




class TestCombinations(unittest.TestCase):
    def setUp(self):
        path_constants = room(254)
        for path in path_constants:
            if not os.path.exists(path):
                self.fail(
                    f"{path} does not exist on your system. If you would like to run tests, please change this path.")

        self.hplc_controller = HPLCController(comm_dir=path_constants[0],
                                              method_dir=path_constants[1],
                                              data_dir=path_constants[2],
                                              sequence_dir=path_constants[3])

    def test_run_method_after_update(self):
        self.hplc_controller.method_controller.switch(DEFAULT_METHOD)

        try:
            rand_method = gen_rand_method()
            self.hplc_controller.edit_method(rand_method)
            if run_too:
                self.hplc_controller.run_method(experiment_name="changed_method")
        except Exception as e:
            self.fail(f"Should have not failed: {e}")

    def test_run_after_table_edit(self):
        try:
            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
            seq_table = self.hplc_controller.load_sequence()
            seq_table.rows[0].vial_location = TenColumn.FIVE
            seq_table.rows[1].vial_location = TenColumn.ONE
            seq_table.rows[0].inj_source = InjectionSource.HIP_ALS
            seq_table.rows[1].inj_source = InjectionSource.MANUAL
            self.hplc_controller.edit_sequence(seq_table)
            if run_too:
                self.hplc_controller.run_sequence(seq_table)
                chrom = self.hplc_controller.get_last_run_sequence_data()
                self.assertTrue(len(chrom) == 2)
        except Exception as e:
            self.fail("Failed")

    def test_run_after_existing_row_edit(self):
        try:
            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
            seq_table = self.hplc_controller.load_sequence()
            self.hplc_controller.edit_sequence_row(seq_entry, 1)
            if run_too:
                self.hplc_controller.run_sequence(seq_table)
                chrom = self.hplc_controller.get_last_run_sequence_data()
                self.assertTrue(len(chrom) == 2)
        except Exception:
            self.fail("Failed")

    def test_update_method_update_seq_table_run(self):
        try:
            self.hplc_controller.method_controller.switch(DEFAULT_METHOD)
            rand_method = gen_rand_method()
            self.hplc_controller.edit_method(rand_method, save=True)

            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
            seq_table = self.hplc_controller.load_sequence()
            seq_table.rows[0].vial_location = TenColumn.ONE
            seq_table.rows[0].inj_source = InjectionSource.HIP_ALS
            seq_table.rows[1].vial_location = TenColumn.TWO
            seq_table.rows[1].inj_source = InjectionSource.HIP_ALS
            self.hplc_controller.edit_sequence(seq_table)

            if run_too:
                self.hplc_controller.run_sequence(seq_table)
                chrom = self.hplc_controller.get_last_run_sequence_data()
                self.assertTrue(len(chrom) == 2)
        except Exception:
            self.fail("Failed")

    def test_update_table_then_row(self):
        try:
            self.hplc_controller.method_controller.switch(DEFAULT_METHOD)
            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
            rand_method = gen_rand_method()
            self.hplc_controller.edit_method(rand_method, save=True)

            seq_table = self.hplc_controller.load_sequence()
            seq_table.rows[0].vial_location = TenColumn.ONE
            seq_table.rows[0].inj_source = InjectionSource.HIP_ALS
            seq_table.rows[0].method = DEFAULT_METHOD
            seq_table.rows[1].vial_location = TenColumn.TWO
            seq_table.rows[1].inj_source = InjectionSource.HIP_ALS
            seq_table.rows[1].method = DEFAULT_METHOD

            self.hplc_controller.edit_sequence(seq_table)
            self.hplc_controller.edit_sequence_row(
                SequenceEntry(
                    vial_location=TenColumn.ONE,
                    method=DEFAULT_METHOD,
                    num_inj=3,
                    inj_vol=4,
                    sample_name="Blank",
                    sample_type=SampleType.BLANK,
                )
            )
            if run_too:
                self.hplc_controller.run_sequence(seq_table)
                chrom = self.hplc_controller.get_last_run_sequence_data()
                self.assertTrue(len(chrom) == 2)
        except Exception:
            self.fail("Failed")

    def test_run_after_new_row_edit(self):
        try:
            self.hplc_controller.switch_sequence(sequence_name=DEFAULT_SEQUENCE)
            seq_table = self.hplc_controller.load_sequence()
            self.hplc_controller.edit_sequence_row(seq_entry, 3)
            if run_too:
                self.hplc_controller.run_sequence(seq_table)
                chrom = self.hplc_controller.get_last_run_sequence_data()
                self.assertTrue(len(chrom) == 2)
        except Exception:
            self.fail("Failed")



from typing import Any

from copy import deepcopy

import os
import time

from .table_controller import TableController
from ...control import CommunicationController
from ...utils.chromatogram import SEQUENCE_TIME_FORMAT, AgilentHPLCChromatogram
from ...utils.macro import Command
from ...utils.sequence_types import SequenceTable, SequenceEntry, SequenceDataFiles, InjectionSource, SampleType
from ...utils.table_types import TableOperation, RegisterFlag, Table
from ...utils.tray_types import TenColumn


class SequenceController(TableController):
    """
    Class containing sequence related logic
    """

    def __init__(self, controller: CommunicationController, src: str, data_dir: str, table: Table, method_dir: str):
        self.method_dir = method_dir
        super().__init__(controller, src, data_dir, table)

    def load(self) -> SequenceTable:
        rows = self.get_num_rows()
        self.send(Command.GET_SEQUENCE_CMD)
        seq_name = self.receive()

        if rows.is_ok() and seq_name.is_ok():
            return SequenceTable(
                name=seq_name.ok_value.num_response,
                rows=[self.get_row(r + 1) for r in range(int(rows.ok_value.num_response))])
        raise RuntimeError(rows.err_value)

    def get_row(self, row: int) -> SequenceEntry:
        sample_name = self.get_text(row, RegisterFlag.NAME)
        vial_location = int(self.get_num(row, RegisterFlag.VIAL_LOCATION))
        method = self.get_text(row, RegisterFlag.METHOD)
        num_inj = int(self.get_num(row, RegisterFlag.NUM_INJ))
        inj_vol = int(self.get_text(row, RegisterFlag.INJ_VOL))
        inj_source = InjectionSource(self.get_text(row, RegisterFlag.INJ_SOR))
        sample_type = SampleType(self.get_num(row, RegisterFlag.SAMPLE_TYPE))
        return SequenceEntry(sample_name=sample_name,
                             vial_location=vial_location,
                             method=None if len(method) == 0 else method,
                             num_inj=num_inj,
                             inj_vol=inj_vol,
                             inj_source=inj_source,
                             sample_type=sample_type, )

    def switch(self, seq_name: str):
        """
        Switch to the specified sequence. The sequence name does not need the '.S' extension.

        :param seq_name: The name of the sequence file
        """
        self.send(f'_SeqFile$ = "{seq_name}.S"')
        self.send(f'_SeqPath$ = "{self.src}"')
        self.send(Command.SWITCH_SEQUENCE_CMD)
        time.sleep(2)
        self.send(Command.GET_SEQUENCE_CMD)
        time.sleep(2)
        parsed_response = self.receive().value.string_response

        assert parsed_response == f"{seq_name}.S", "Switching sequence failed."

    def edit(self, sequence_table: SequenceTable):
        """
        Updates the currently loaded sequence table with the provided table. This method will delete the existing sequence table and remake it.
        If you would only like to edit a single row of a sequence table, use `edit_sequence_table_row` instead.

        :param sequence_table:
        """
        self.send("Local Rows")
        self.sleep(1)
        self.delete_table()
        self.sleep(1)
        self.new_table()
        self.sleep(1)
        self.get_num_rows()

        if True:
            for _ in sequence_table.rows:
                self.add_row()
                self.sleep(1)
                self.send(Command.SAVE_SEQUENCE_CMD)
            self.get_num_rows()
            self.send(Command.SAVE_SEQUENCE_CMD)
            for i, row in enumerate(sequence_table.rows):
                self.edit_row(row=row, row_num=i + 1)
                self.sleep(1)
            self.send(Command.SAVE_SEQUENCE_CMD)
        else:
            for i, row in enumerate(sequence_table.rows):
                self.add_row()
                self.sleep(0.1)
                self.edit_row(row=row, row_num=i + 1)
                self.sleep(0.1)
                rows = self.get_num_rows()
                self.send(Command.SAVE_SEQUENCE_CMD)
                self.send(Command.SWITCH_SEQUENCE_CMD)

    def edit_row(self, row: SequenceEntry, row_num: int):
        """
        Edits a row in the sequence table. If a row does NOT exist, a new one will be created.

        :param row: sequence row entry with updated information
        :param row_num: the row to edit, based on 1-based indexing
        """
        num_rows = self.get_num_rows()
        if num_rows.is_ok():
            while num_rows.ok_value.num_response < row_num:
                self.add_row()
                self.send(Command.SAVE_SEQUENCE_CMD)
                num_rows = self.get_num_rows()

        table_register = self.table.register
        table_name = self.table.name

        if row.vial_location:
            self.sleepy_send(TableOperation.EDIT_ROW_VAL.value.format(register=table_register,
                                                                      table_name=table_name,
                                                                      row=row_num,
                                                                      col_name=RegisterFlag.VIAL_LOCATION,
                                                                      val=row.vial_location.value))
        if row.method:
            possible_path = os.path.join(self.method_dir, row.method)
            method = row.method
            if os.path.exists(possible_path):
                method = os.path.join(self.method_dir, row.method)
            self.sleepy_send(TableOperation.EDIT_ROW_TEXT.value.format(register=table_register,
                                                                       table_name=table_name,
                                                                       row=row_num,
                                                                       col_name=RegisterFlag.METHOD,
                                                                       val=method))

        if row.num_inj:
            self.sleepy_send(TableOperation.EDIT_ROW_VAL.value.format(register=table_register,
                                                                      table_name=table_name,
                                                                      row=row_num,
                                                                      col_name=RegisterFlag.NUM_INJ,
                                                                      val=row.num_inj))

        if row.inj_vol:
            self.sleepy_send(TableOperation.EDIT_ROW_TEXT.value.format(register=table_register,
                                                                       table_name=table_name,
                                                                       row=row_num,
                                                                       col_name=RegisterFlag.INJ_VOL,
                                                                       val=row.inj_vol))

        if row.inj_source:
            self.sleepy_send(TableOperation.EDIT_ROW_TEXT.value.format(register=table_register,
                                                                       table_name=table_name,
                                                                       row=row_num,
                                                                       col_name=RegisterFlag.INJ_SOR,
                                                                       val=row.inj_source.value))

        if row.sample_name:
            self.sleepy_send(TableOperation.EDIT_ROW_TEXT.value.format(register=table_register,
                                                                       table_name=table_name,
                                                                       row=row_num,
                                                                       col_name=RegisterFlag.NAME,
                                                                       val=row.sample_name))
            self.sleepy_send(TableOperation.EDIT_ROW_TEXT.value.format(register=table_register,
                                                                       table_name=table_name,
                                                                       row=row_num,
                                                                       col_name=RegisterFlag.DATA_FILE,
                                                                       val=row.sample_name))
        if row.sample_type:
            self.sleepy_send(TableOperation.EDIT_ROW_VAL.value.format(register=table_register,
                                                                      table_name=table_name,
                                                                      row=row_num,
                                                                      col_name=RegisterFlag.SAMPLE_TYPE,
                                                                      val=row.sample_type.value))

        self.send(Command.SAVE_SEQUENCE_CMD)

    def run(self, sequence_table: SequenceTable):
        """
        Starts the currently loaded sequence, storing data
        under the <data_dir>/<sequence table name> folder.
        Device must be ready.

        :param sequence_table:
        """
        timestamp = time.strftime(SEQUENCE_TIME_FORMAT)
        self.send(Command.RUN_SEQUENCE_CMD.value)

        if self.check_hplc_is_running():
            folder_name = f"{sequence_table.name} {timestamp}"
            self.data_files.append(SequenceDataFiles(dir=folder_name,
                                                     sequence_name=sequence_table.name, ))

        run_completed = self.check_hplc_done_running()

        if run_completed.is_ok():
            self.data_files[-1].dir = run_completed.value
        else:
            raise RuntimeError("Run error has occured.")

    def retrieve_recent_data_files(self):
        sequence_data_files: SequenceDataFiles = self.data_files[-1]
        return os.path.join(sequence_data_files.dir, sequence_data_files.child_dirs[-1])

    def get_data(self) -> list[dict[str, AgilentHPLCChromatogram]]:
        sequence_data_files: SequenceDataFiles = self.data_files[-1]
        subdirs = [x[0] for x in os.walk(self.data_dir)]
        potential_folders = sorted(list(filter(lambda d: sequence_data_files.dir in d, subdirs)))
        self.data_files[-1].child_dirs = potential_folders
        spectra: list[dict[str, AgilentHPLCChromatogram]] = []
        for row in sequence_data_files.child_dirs:
            data_path = os.path.join(sequence_data_files.dir, row)
            self.get_spectrum(data_path)
            spectra.append(deepcopy(self.spectra))
        return spectra



"""
Module to provide API for higher-level HPLC actions.

Authors: Lucy Hao
"""

from typing import Union, Optional

from ..control import CommunicationController
from ..control.table import MethodController, SequenceController
from ..utils.chromatogram import AgilentHPLCChromatogram
from ..utils.macro import Command, HPLCRunningStatus, HPLCAvailStatus, HPLCErrorStatus, Response
from ..utils.method_types import MethodTimetable
from ..utils.sequence_types import SequenceTable, SequenceEntry
from ..utils.table_types import Table


class HPLCController:
    # tables
    METHOD_TIMETABLE = Table(
        register="RCPMP1Method[1]",
        name="Timetable"
    )

    SEQUENCE_TABLE = Table(
        register="_sequence[1]",
        name="SeqTable1"
    )

    def __init__(self,
                 comm_dir: str,
                 data_dir: str,
                 method_dir: str,
                 sequence_dir: str):
        """Initialize HPLC controller. The `hplc_talk.mac` macro file must be loaded in the Chemstation software.
        `comm_dir` must match the file path in the macro file.

        :param comm_dir: Name of directory for communication, where ChemStation will read and write from. Can be any existing directory.
        :raises FileNotFoundError: If either `data_dir`, `method_dir` or `comm_dir` is not a valid directory.
        """
        self.comm = CommunicationController(comm_dir=comm_dir)
        self.method_controller = MethodController(controller=self.comm,
                                                  src=method_dir,
                                                  data_dir=data_dir,
                                                  table=self.METHOD_TIMETABLE)
        self.sequence_controller = SequenceController(controller=self.comm,
                                                      src=sequence_dir,
                                                      data_dir=data_dir,
                                                      table=self.SEQUENCE_TABLE,
                                                      method_dir=method_dir)

    def send(self, cmd: Union[Command, str]):
        self.comm.send(cmd)

    def receive(self) -> Response:
        return self.comm.receive().value

    def status(self) -> Union[HPLCRunningStatus | HPLCAvailStatus | HPLCErrorStatus]:
        return self.comm.get_status()

    def switch_method(self, method_name: str):
        """
        Allows the user to switch between pre-programmed methods. No need to append '.M'
        to the end of the method name. For example. for the method named 'General-Poroshell.M',
        only 'General-Poroshell' is needed.

        :param method_name: any available method in Chemstation method directory
        :raise IndexError: Response did not have expected format. Try again.
        :raise AssertionError: The desired method is not selected. Try again.
        """
        self.method_controller.switch(method_name)

    def switch_sequence(self, sequence_name: str):
        """
         Allows the user to switch between pre-programmed sequences. The sequence name does not need the '.S' extension.
         For example. for the method named 'mySeq.S', only 'mySeq' is needed.

        :param sequence_name: The name of the sequence file
        """
        self.sequence_controller.switch(sequence_name)

    def run_method(self, experiment_name: str):
        """
        This is the preferred method to trigger a run.
        Starts the currently selected method, storing data
        under the <data_dir>/<experiment_name>.D folder.
        The <experiment_name> will be appended with a timestamp in the '%Y-%m-%d-%H-%M' format.
        Device must be ready.

        :param experiment_name: Name of the experiment
        """
        self.method_controller.run(experiment_name)

    def run_sequence(self, sequence_table: SequenceTable):
        """
        Starts the currently loaded sequence, storing data
        under the <data_dir>/<sequence table name> folder.
        Device must be ready.

        :param sequence_table:
        """
        self.sequence_controller.run(sequence_table)

    def edit_method(self, updated_method: MethodTimetable, save: bool = False):
        """Updated the currently loaded method in ChemStation with provided values.

        :param updated_method: the method with updated values, to be sent to Chemstation to modify the currently loaded method.
        :param save:
        """
        self.method_controller.edit(updated_method, save)

    def edit_sequence(self, updated_sequence: SequenceTable):
        """
        Updates the currently loaded sequence table with the provided table. This method will delete the existing sequence table and remake it.
        If you would only like to edit a single row of a sequence table, use `edit_sequence_row` instead.

        :param sequence_table:
        """
        self.sequence_controller.edit(updated_sequence)

    def edit_sequence_row(self, row: SequenceEntry, num: int):
        """
        Edits a row in the sequence table. Assumes the row already exists.

        :param row: sequence row entry with updated information
        :param row_num: the row to edit, based on -1-based indexing
        """
        self.sequence_controller.edit_row(row, num)

    def get_last_run_method_data(self) -> dict[str, AgilentHPLCChromatogram]:
        """
        Returns the last run method data.
        """
        return self.method_controller.get_data()

    def get_last_run_sequence_data(self) -> list[dict[str, AgilentHPLCChromatogram]]:
        """
        Returns data for all rows in the last run sequence data.
        """
        return self.sequence_controller.get_data()

    def load_method(self) -> MethodTimetable:
        """
        Returns the currently loaded method, including its timetable.
        """
        return self.method_controller.load()

    def load_sequence(self) -> SequenceTable:
        """
        Returns the currently loaded sequence.
        """
        return self.sequence_controller.load()

    def standby(self):
        """Switches all modules in standby mode. All lamps and pumps are switched off."""
        self.send(Command.STANDBY_CMD)

    def preprun(self):
        """ Prepares all modules for run. All lamps and pumps are switched on."""
        self.send(Command.PREPRUN_CMD)

    def lamp_on(self):
        """Turns the UV lamp on."""
        self.send(Command.LAMP_ON_CMD)

    def lamp_off(self):
        """Turns the UV lamp off."""
        self.send(Command.LAMP_OFF_CMD)

    def pump_on(self):
        """Turns on the pump on."""
        self.send(Command.PUMP_ON_CMD)

    def pump_off(self):
        """Turns the pump off."""
        self.send(Command.PUMP_OFF_CMD)



from dataclasses import dataclass
from enum import Enum


class TableOperation(Enum):
    def __str__(self):
        return '%s' % self.value

    DELETE_TABLE = 'DelTab {register}, "{table_name}"'
    CREATE_TABLE = 'NewTab {register}, "{table_name}"'
    NEW_ROW = 'InsTabRow {register}, "{table_name}"'
    EDIT_ROW_VAL = 'SetTabVal "{register}", "{table_name}", {row}, "{col_name}", {val}'
    EDIT_ROW_TEXT = 'SetTabText "{register}", "{table_name}", {row}, "{col_name}", "{val}"'
    GET_ROW_VAL = 'TabVal("{register}", "{table_name}", {row}, "{col_name}")'
    GET_ROW_TEXT = 'TabText$("{register}", "{table_name}", {row}, "{col_name}")'
    GET_NUM_ROWS = 'Rows = TabHdrVal({register}, "{table_name}", "{col_name}")'
    GET_OBJ_HDR_VAL = 'ObjHdrVal("{register}", "{register_flag}")'
    GET_OBJ_HDR_TEXT = 'ObjHdrText$("{register}", "{register_flag}")'
    UPDATE_OBJ_HDR_VAL = 'SetObjHdrVal {register}, {register_flag}, {val}'
    UPDATE_OBJ_HDR_TEXT = 'SetObjHdrText {register}, {register_flag}, {val}'
    NEW_COL_TEXT = 'NewColText {register}, "{table_name}", "{col_name}", "{val}"'
    NEW_COL_VAL = 'NewColVal {register}, "{table_name}", "{col_name}", {val}'


class RegisterFlag(Enum):
    def __str__(self):
        return '%s' % self.value

    # for table
    NUM_ROWS = "NumberOfRows"

    # for Method
    SOLVENT_A_COMPOSITION = "PumpChannel_CompositionPercentage"
    SOLVENT_B_COMPOSITION = "PumpChannel2_CompositionPercentage"
    SOLVENT_C_COMPOSITION = "PumpChannel3_CompositionPercentage"
    SOLVENT_D_COMPOSITION = "PumpChannel4_CompositionPercentage"
    FLOW = "Flow"
    MAX_TIME = "StopTime_Time"
    COLUMN_OVEN_TEMP1 = "TemperatureControl_Temperature"
    COLUMN_OVEN_TEMP2 = "TemperatureControl2_Temperature"
    STOPTIME_MODE = "StopTime_Mode"
    POSTIME_MODE = "PostTime_Mode"
    TIME = "Time"
    TIMETABLE_SOLVENT_B_COMPOSITION = "SolventCompositionPumpChannel2_Percentage"

    # for Method Timetable
    SOLVENT_COMPOSITION = "SolventComposition"

    # for Sequence
    VIAL_LOCATION = "Vial"
    NAME = "SampleName"
    METHOD = "Method"
    INJ_VOL = "InjVolume"
    INJ_SOR = "InjectionSource"
    NUM_INJ = "InjVial"
    SAMPLE_TYPE = "SampleType"
    DATA_FILE = "DataFileName"


@dataclass
class Table:
    register: str
    name: str
